<?php

$pm_recently_viewed_admin_obj = pm_recently_viewed_admin::get_instance();

add_action('admin_init', array($pm_recently_viewed_admin_obj, 'register_admin_settings'));
add_action('admin_init', array($pm_recently_viewed_admin_obj, 'add_buttons'));
add_action('admin_menu', array($pm_recently_viewed_admin_obj, 'setup_admin_menu'));

/**
 * Description of admin
 *
 * @author User
 */
class pm_recently_viewed_admin {
    /**
     * Singleton pattern i.e. we have only one instance of this obj
     * 
     * @staticvar type $instance
     * @return \cls
     */
    public static function get_instance() {
        static $instance = null;

        if (is_null($instance)) {
            $cls = __CLASS__;
            $instance = new $cls();
        }

        return $instance;
    }

    private $plugin_tinymce_name = 'pm_recently_viewed'; // i
    
    /**
     * Adds buttons only for RichText mode
     * @return void
     */
    function add_buttons() {
        // Don't bother doing this stuff if the current user lacks permissions
        if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
            return;
        }

        // Add only in Rich Editor mode
        if ( get_user_option('rich_editing') == 'true' ) {
            // add the button for wp2.5 in a new way
            add_filter( 'mce_buttons', array( $this, 'register_button' ), 5 );
            add_filter( "mce_external_plugins", array( $this, "add_tinymce_plugin" ), 5);

            // Required by TinyMCE button
            add_action( 'wp_ajax_pm_recently_viewed_ajax_render_popup_content', array( $this, 'ajax_render_popup_content' ) );
        }
    }

    /**
     * This is triggered by editor_plugin.min.js and WP proxies the ajax calls to this action.
     *
     * @return void
     */
    function ajax_render_popup_content() {
        if ( ! is_user_logged_in() ) { // check for rights. This shouldn't be necessary as we don't handle nopriv ajax call.
            wp_die(__("You must be logged in order to use this plugin."));
        }

        $site_url = site_url();

        ?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml">
            <head>
                <title>PM Recently Viewed</title>
                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
                <script language="javascript" type="text/javascript" src="<?php echo $site_url; ?>/wp-includes/js/tinymce/tiny_mce_popup.js"></script>
                <script language="javascript" type="text/javascript" src="<?php echo $site_url; ?>/wp-includes/js/tinymce/utils/mctabs.js"></script>
                <script language="javascript" type="text/javascript" src="<?php echo $site_url; ?>/wp-includes/js/tinymce/utils/form_utils.js"></script>

                <script language="javascript" type="text/javascript">
                    function pm_recently_viewed_js_init() {
                        tinyMCEPopup.resizeToInnerSize();
                    }

                    function pm_recently_viewed_js_insert_shortcode() {
                        var content = '';
                        var attribs_str = '';
                        var template_recently_viewed = '<br/><p>[pm_recently_viewed %%ATTRIBS%%]</p><br/>';
                        var template_related = '<br/><p>[pm_recently_viewed_related %%ATTRIBS%%]</p><br/>';

                        var pm_recently_viewed = document.getElementById('pm_recently_viewed_panel');

                        // who is active ?
                        if (pm_recently_viewed.className.indexOf('current') != -1) {
                            var limits = document.getElementById('limits').value;
                            limits = parseInt(limits);

                            if ( limits > 0 ) {
                                attribs_str += ' limit="' + limits + '"';
                            }

                            var label = document.getElementById('pm_recently_viewed_label').value;

                            if ( label != '' ) {
                                attribs_str += ' label="' + label + '"';
                            }

                            content = document.getElementById('pm_recently_viewed_type').checked
                                ? template_recently_viewed
                                : template_related;

                            content = content.replace(/\s*%%ATTRIBS%%/ig, attribs_str);
                        }

                        if (window.tinyMCE) {
                            /* get the TinyMCE version to account for API diffs */
                            // @see http://stackoverflow.com/questions/22813970/typeerror-window-tinymce-execinstancecommand-is-not-a-function
                            var tmce_ver = window.tinyMCE.majorVersion;

                            if (tmce_ver >= "4") {
                                window.tinyMCE.execCommand('mceInsertContent', false, content);
                            } else {
                                window.tinyMCE.execInstanceCommand('content', 'mceInsertContent', false, content);
                            }

                            //Peforms a clean up of the current editor HTML.
                            //tinyMCEPopup.editor.execCommand('mceCleanup');
                            //Repaints the editor. Sometimes the browser has graphic glitches.
                            tinyMCEPopup.editor.execCommand('mceRepaint');
                            tinyMCEPopup.close();
                        }

                        return;
                    }
                </script>
                <style>
                    body {
                        color: #444;
                        font-family: "Open Sans",sans-serif;
                        font-size: 13px;
                        line-height: 1.4em;
                    }

                    .tabs a {
                        font-size: 13px;
                    }

                    .pm_recently_viewed .app_positive_button {
                        background:#99CC66 !important;
                        color: black !important;
                    }

                    .pm_recently_viewed .app_negative_button {
                        background:#F19C96 !important;
                        color: black !important;
                    }

                    .pm_recently_viewed .app_max_width {
                        width: 100%;
                    }

                    .pm_recently_viewed .app_text_field {
                        border: 1px solid #888888;
                        padding: 3px;
                    }
                </style>
                <base target="_self" />
            </head>
            <body id="pm_recently_viewed" class="pm_recently_viewed"
                  onload="tinyMCEPopup.executeOnLoad('pm_recently_viewed_js_init();');
                        document.body.style.display = '';"
                  style="display: none">
                <form name="pm_recently_viewed_form" action="#">
                    <div class="tabs">
                        <ul>
                            <li id="pm_recently_viewed_tab" class="current"><span><a href="javascript:mcTabs.displayTab('pm_recently_viewed_tab','pm_recently_viewed_panel');" onmousedown="return false;"><?php _e("PM Recently Viewed", 'pm_recently_viewed'); ?></a></span></li>
                        </ul>
                    </div>

                    <div class="panel_wrapper">
                        <!-- panel -->
                        <div id="pm_recently_viewed_panel" class="panel current">
                            <br />
                            <div>
                                Please fill out the fields and click insert to insert the shortcode into the post/page.
                            </div>
                            <br/>
                            <table border="0" cellpadding="4" cellspacing="0" class="app_max_width">
                                <tr>
                                    <td nowrap="nowrap">
                                        <label for="pm_recently_viewed_product_name"><?php _e("Type", 'pm_recently_viewed'); ?></label>
                                    </td>
                                    <td>
                                        <label for="pm_recently_viewed_type">
                                            <input type="radio" id='pm_recently_viewed_type' name='pm_recently_viewed_type' value="recently_viewed" checked="checked" /> Recently Viewed
                                        </label>
                                        <br />

                                        <label for="pm_recently_viewed_type2">
                                            <input type="radio" id='pm_recently_viewed_type2' name='pm_recently_viewed_type' value="related" /> Related
                                        </label>
                                        <br />
                                    </td>
                                </tr>
                                <tr>
                                    <td nowrap="nowrap">
                                        <label for="pm_recently_viewed_label"><?php _e("Label", 'pm_recently_viewed'); ?></label>
                                    </td>
                                    <td>
                                        <input type="text" id='pm_recently_viewed_label' class="app_max_width" name='pm_recently_viewed_label' value="" placeholder="Text shown before items (optional)" />
                                    </td>
                                </tr>
                                <tr>
                                    <td nowrap="nowrap">
                                        <label for="pm_recently_viewed_product_name"><?php _e("Items per page", 'pm_recently_viewed'); ?></label>
                                    </td>
                                    <td>
                                        <input type="number" id='limits' name='limits' placeholder="Limits (optional)" />
                                    </td>
                                </tr>
                            </table>
                        </div>
                        <!-- end panel -->
                    </div>

                    <div class="mceActionPanel">
                        <div style="float: left">
                            <input type="submit" id="insert" name="insert" value="<?php _e("Insert", 'pm_recently_viewed'); ?>"
                                   class='app_positive_button'
                                   onclick="pm_recently_viewed_js_insert_shortcode(); return false;" />
                        </div>

                        <div style="float: right">
                            <input type="button" id="cancel" name="cancel" value="<?php _e("Cancel", 'pm_recently_viewed'); ?>"
                                   class='app_negative_button'
                                   onclick="tinyMCEPopup.close();" />
                        </div>
                    </div>
                </form>
            </body>
        </html>
        <?php
        die(); // This is required to return a proper result
    }



    // used to insert button in wordpress 2.5x editor
    function register_button($buttons) {
        array_push($buttons, "separator", $this->plugin_tinymce_name);
        return $buttons;
    }

    // Load the TinyMCE plugin : editor_plugin.js (wp2.5)
    function add_tinymce_plugin($plugin_array) {
        $plugin_array[$this->plugin_tinymce_name] = plugins_url('/modules/tinymce/editor_plugin.js', PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE);
        return $plugin_array;
    }

    /**
     * We don't want any assholes using the site.
     */
    public function register_admin_settings() {
        register_setting('pm_recently_viewed_settings', 'pm_recently_viewed_options', array($this, 'validate_settings'));
    }

    /**
     * This is called by WP after the user hits the submit button.
     * The variables are trimmed first and then passed to the who ever wantsto filter them.
     * @param array the entered data from the settings page.
     * @return array the modified input array
     * @todo force numbers for 2 params
     */
    function validate_settings($input) { // whitelist options
        $input = array_map('trim', $input);

        // let extensions do their thing
        $input_filtered = apply_filters('pm_recently_viewed_admin_filter_settings', $input);

        // did the extension break stuff?
        $input = is_array($input_filtered) ? $input_filtered : $input;

        return $input;
    }

    /**
     * Set up administration
     *
     * @package PM Recently Viewed
     * @since 0.1
     */
    public function setup_admin_menu() {
        $hook = add_options_page(PM_RECENTLY_VIEWED_PLUGIN_NAME, PM_RECENTLY_VIEWED_PLUGIN_NAME, 'manage_options', PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE, array($this, 'output_options_page'));

        add_filter('plugin_action_links', array($this, 'add_quick_settings_link'), 10, 2);
    }

    /**
     * Adds the action link to settings. That's from Plugins. It is a nice thing.
     * @param type $links
     * @param type $file
     * @return type
     */
    function add_quick_settings_link($links, $file) {
        if ($file == plugin_basename(PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE)) {
            $link = admin_url('options-general.php?page=' . plugin_basename(PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE));
            $html_link = "<a href=\"{$link}\">Settings</a>";
            array_unshift($links, $html_link);
        }

        return $links;
    }

    /**
     * Retrieves the plugin options. It inserts some defaults.
     * The saving is handled by the settings page. Basically, we submit to WP and it takes
     * care of the saving.
     *
     * @return array
     */
    function get_options() {
        $defaults = array(
            'recently_viewed_label' => 'Recently Viewed',
            'recently_viewed_limit' => 3,
            'related_posts_limit' => 3,

            /*
                linked_title_only
                linked_title_with_featured_image
                linked_title_with_excerpt
                linked_title_with_featured_image_and_excerpt
             */
            'how_to_render' => 'linked_title_only',
        );

        $opts = get_option('pm_recently_viewed_options');

        $opts = (array) $opts;
        $opts = array_merge($defaults, $opts);

        return $opts;
    }

    /**
     * Options page
     *
     * @package PM Recently Viewed
     * @since 1.0
     */
    function output_options_page() {
        $opts = $this->get_options();
        ?>

        <div class="wrap pm_recently_viewed_admin_wrapper pm_recently_viewed_container">

            <div id="icon-options-general" class="icon32"></div>
            <h2><?php echo PM_RECENTLY_VIEWED_PLUGIN_NAME; ?></h2>

            <div id="poststuff">

                <div id="post-body" class="metabox-holder columns-2">

                    <!-- main content -->
                    <div id="post-body-content">

                        <div class="meta-box-sortables ui-sortable">

                            <div class="postbox">

                                <!--<h3><span>Settings</span></h3>-->
                                <div class="inside">
                                    <?php if (1) : ?>
                                        <form method="post" action="options.php">
                                            <?php settings_fields('pm_recently_viewed_settings'); ?>
                                            <table class="form-table">

                                                <tr valign="top">
                                                    <th scope="row">Recently Viewed Limit </th>
                                                    <td>
                                                        <label for="pm_recently_viewed_options_recently_viewed_limit">
                                                            <input type="number" id="pm_recently_viewed_options_recently_viewed_limit"
                                                                   name="pm_recently_viewed_options[recently_viewed_limit]"
                                                                   value="<?php echo $opts['recently_viewed_limit']; ?>" />
                                                        </label>
                                                    </td>
                                                </tr>

                                                <tr valign="top">
                                                    <th scope="row">Related Posts Limit </th>
                                                    <td>
                                                        <label for="pm_recently_viewed_options_related_posts_limit">
                                                             <input type="number" id="pm_recently_viewed_options_related_posts_limit"
                                                                   name="pm_recently_viewed_options[related_posts_limit]"
                                                                   value="<?php echo $opts['related_posts_limit']; ?>" />
                                                        </label>
                                                    </td>
                                                </tr>

                                                <?php if (0) : ?>
                                                <tr valign="top">
                                                    <th scope="row">Recently Viewed Label </th>
                                                    <td>
                                                        <label for="pm_recently_viewed_options_recently_viewed_label">
                                                            <input type="text" id="pm_recently_viewed_options_recently_viewed_label"
                                                                   name="pm_recently_viewed_options[recently_viewed_label]"
                                                                   value="<?php echo $opts['recently_viewed_label']; ?>" />
                                                        </label>
                                                    </td>
                                                </tr>
                                                <?php endif; ?>

                                                <tr valign="top">
                                                    <th scope="row">How to Render Recently Viewed</th>
                                                    <td>
                                                        <label for="linked_title_only">
                                                            <input type="radio" id="linked_title_only" name="pm_recently_viewed_options[how_to_render]"
                                                                   value="linked_title_only" <?php echo checked($opts['how_to_render'], 'linked_title_only'); ?> />
                                                            Linked title only
                                                        </label>
                                                        <br/>

                                                        <label for="linked_title_with_featured_image">
                                                            <input type="radio" id="linked_title_with_featured_image" name="pm_recently_viewed_options[how_to_render]"
                                                                   value="linked_title_with_featured_image" <?php echo checked($opts['how_to_render'], 'linked_title_with_featured_image'); ?> />
                                                            Linked title with featured image
                                                        </label>
                                                        <br/>

                                                        <label for="linked_title_with_excerpt">
                                                            <input type="radio" id="linked_title_with_excerpt" name="pm_recently_viewed_options[how_to_render]"
                                                                   value="linked_title_with_excerpt" <?php echo checked($opts['how_to_render'], 'linked_title_with_excerpt'); ?> />
                                                            Linked title with excerpt
                                                        </label>
                                                        <br/>

                                                        <label for="linked_title_with_featured_image_and_excerpt">
                                                            <input type="radio" id="linked_title_with_featured_image_and_excerpt" name="pm_recently_viewed_options[how_to_render]"
                                                                   value="linked_title_with_featured_image_and_excerpt" <?php echo checked($opts['how_to_render'], 'linked_title_with_featured_image_and_excerpt'); ?> />
                                                            Linked title with featured image and excerpt
                                                        </label>
                                                    </td>
                                                </tr>
                                            </table>

                                            <p class="submit">
                                                <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                                            </p>
                                        </form>
                                    <?php else : ?>
                                        <div>
                                            The plugin doesn't have settings options at the moment.<br/><br/>
                                        </div>

                                    <?php endif; ?>
                                </div> <!-- .inside -->
                            </div> <!-- .postbox -->

                        </div> <!-- .meta-box-sortables .ui-sortable -->

                    </div> <!-- post-body-content -->

                </div> <!-- #post-body .metabox-holder .columns-2 -->

                <br class="clear">
            </div> <!-- #poststuff -->

        </div> <!-- .wrap -->
        <?php
    }
}
