<?php

$pm_front = new pm_recently_viewed_front();

add_action( 'init', array( $pm_front, 'init' ) );
add_action( 'wp', array( $pm_front, 'track_viewed_page' ) );

class pm_recently_viewed_front {
    /**
     * We don't want any assholes using the site.
     */
    public function init() {
        @session_start();

        if ( isset( $_REQUEST['clear_pm_recently_viewed_pages'] ) ) {
            $r = remove_query_arg( 'clear_pm_recently_viewed_pages' );
            $_SESSION[ 'pm_recently_viewed_pages' ] = array();

            $msg = PM_RECENTLY_VIEWED_PLUGIN_NAME . " session data has been cleared. <br/>\n";
            $msg .= sprintf( "<a href='%s'>Continue</a>", $r );

            wp_die( $msg, PM_RECENTLY_VIEWED_PLUGIN_NAME, array( 'response' => 200 ) );
        }

        $_SESSION['pm_recently_viewed_pages'] = empty( $_SESSION['pm_recently_viewed_pages'] )
                ? array()
                : $_SESSION['pm_recently_viewed_pages'];

        add_shortcode('pm_recently_viewed', array($this, 'render_recently_viewed'));
        add_shortcode('pm_recently_viewed_related', array($this, 'render_related'));

        $suffix = '';
        wp_register_style('pm_recently_viewed_front', plugins_url("/modules/css/front{$suffix}.css", PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE), false,
            filemtime( plugin_dir_path( PM_RECENTLY_VIEWED_BASE_PLUGIN_FILE ) . "/modules/css/front{$suffix}.css" ) );
        wp_enqueue_style('pm_recently_viewed_front');
    }

    /**
     * Here we don't necessarily need the post id but if it's there that
     * means that we're rendering a post.
     * 
     * @param array $attr
     * @param str $content
     * @return string
     */
    public function render_recently_viewed( $attr = array(), $content = '' ) {
        global $post;

        $post_id = 0;

        // since the widget can be included on the home page we don't necessarily
        // need post id to query the recently viewed ones.
        /*if ( !empty( $attr['id'] ) ) {
            $post_id = $attr['id'];
        } elseif ( !empty( $post->ID ) ) {
            $post_id = $post->ID;
        } else {
            $m = __CLASS__ . ' post id not found so not rendering. no_records ';
            $buff = "<!-- $m -->";
            return;
        }*/
        
        $pm_recently_viewed_admin_obj = pm_recently_viewed_admin::get_instance();
        $opts = $pm_recently_viewed_admin_obj->get_options();

        $display_limit = empty( $attr['limit'] ) ? $opts['recently_viewed_limit'] : $attr['limit'];
        $display_limit = abs( $display_limit );

        $recently_viewed_label = empty( $attr['label'] ) ? '' : $attr['label'];

        $recently_viewed_all = $_SESSION[ 'pm_recently_viewed_pages' ];
        $last_n_posts = array_slice( $recently_viewed_all, ( -1 ) * $display_limit, $display_limit, true );

        $args = array(
            'post_type'      => 'post',
            'posts_per_page' => $display_limit,
            'post__in'       => $last_n_posts, // just to be sure: some claim that one key works and not the other.
            'include'        => $last_n_posts, // I'll try them both just in case.
        );

        $raw_posts = get_posts( $args );
        wp_reset_postdata();

        $posts = $ordered_posts = array();

        foreach ( $raw_posts as $post_obj ) {
            $posts[ $post_obj->ID ] = $this->parse_post( $post_obj );
        }

        // the posts will likely come in a random order so we'll
        // order them in their viewed order.
        // We're looping through saved info and fill a new array
        foreach ( $last_n_posts as $post_id ) {
            $ordered_posts[] = $posts[ $post_id ];
        }

        $render_params = array(
            'label' => $recently_viewed_label,
            'limit' => $display_limit,
        );

        $buff = $this->toHTML( $ordered_posts, $render_params );

        return $buff;
    }

    public function toHTML( $posts, $render_params = array() ) {
        if ( empty( $posts ) ) {
            return "\n<!-- No records no_records no_records -->\n";
        }

        $page_cnt = 0;
        
        $pm_recently_viewed_admin_obj = pm_recently_viewed_admin::get_instance();
        $opts = $pm_recently_viewed_admin_obj->get_options();

        ob_start();
        ?>
        <!-- pm_recently_viewed_page_wrapper -->
        <div id="pm_recently_viewed_page_wrapper" class="pm_recently_viewed_page_wrapper">
            <?php if ( !empty( $render_params['label'] ) ) : ?>
                <h3><?php echo $render_params['label']; ?></h3>
            <?php endif; ?>

            <?php foreach ( $posts as $post_id => $page_rec ) : $page_cnt++; ?>
                <?php if ( $page_cnt > $render_params['limit'] ) { break; } ?>
                <div id='pm_recently_viewed_page_<?php echo $page_rec['id']; ?>' class="pm_recently_viewed_page pm_recently_viewed_page_<?php echo $page_rec['id']; ?>">
                    <div id='pm_recently_viewed_page_<?php echo $page_rec['id']; ?>' class="pm_recently_viewed_page pm_recently_viewed_page_<?php echo $page_rec['id']; ?>">
                        <a href="<?php echo $page_rec['link'];?>" title="<?php echo esc_attr($page_rec['title']); ?>"><?php echo $page_rec['title']; ?></a>
                    </div>

                    <?php if ( in_array( $opts['how_to_render'], array( 'linked_title_with_excerpt', 'linked_title_with_featured_image_and_excerpt' ) ) ) : ?>
                        <!-- excerpt -->
                        <div id='pm_recently_viewed_page_excerpt_<?php echo $page_rec['id']; ?>'
                                class="pm_recently_viewed_page_excerpt pm_recently_viewed_page_excerpt_<?php echo $page_rec['id']; ?>">
                            <?php echo $page_rec['excerpt']; ?>
                        </div> <!-- /excerpt -->
                    <?php endif; ?>

                    <?php if ( in_array( $opts['how_to_render'], array( 'linked_title_with_featured_image', 'linked_title_with_featured_image_and_excerpt' ) ) ) : ?>
                        <!-- featured_image -->
                        <div id='pm_recently_viewed_page_featured_image_<?php echo $page_rec['id']; ?>'
                                class="pm_recently_viewed_page_featured_image pm_recently_viewed_page_featured_image_<?php echo $page_rec['id']; ?>">
                            <?php echo $page_rec['featured_image_html']; ?>
                        </div> <!-- featured_image -->
                    <?php endif; ?>
                </div> <!-- /pm_recently_viewed_page -->
            <?php endforeach; ?>
        </div> <!-- /pm_recently_viewed_page_wrapper -->

        <?php
        $buff = ob_get_contents();
    	ob_end_clean();

        return $buff;
    }

    /**
     * Finds the current post and finds items with the same tags.
     * 
     * @param str $attr
     * @param str $content
     * @return string
     */
    public function render_related( $attr = array(), $content = '' ) {
        global $post;

        $post_id = 0;

        if ( !empty( $attr['id'] ) ) {
            $post_id = $attr['id'];
        } elseif ( !empty( $post->ID ) ) {
            $post_id = $post->ID;
        } else {
            $m = __CLASS__ . ' post id not found so not rendering no_records .';
            $buff = "<!-- $m -->";
            return;
        }

        $tax_name = 'post_tag';
        $term_list = wp_get_post_terms( $post_id, $tax_name, array( "fields" => "all" ) );

        if ( empty( $term_list ) ) {
            $m = __CLASS__ . " the post doesn't have any tags defined. ";
            $buff = "<!-- $m / no_records -->";
            return $buff;
        }
        
        // @see http://wordpress.stackexchange.com/questions/143863/list-posts-by-their-custom-taxonomy-term-with-a-condition
        $slug_array = array(
            'relation' => 'OR',
        );

        foreach ( $term_list as $term_obj ) {
            $slug_array[] = array(
                'taxonomy' => $tax_name,
                'field' => 'slug',
                'terms' => $term_obj->slug,
            );
        }

        $pm_recently_viewed_admin_obj = pm_recently_viewed_admin::get_instance();
        $opts = $pm_recently_viewed_admin_obj->get_options();

        $display_limit = empty( $attr['limit'] ) ? $opts['related_posts_limit'] : $attr['limit'];
        $display_limit = abs( $display_limit );

        $related_label = empty( $attr['label'] ) ? '' : $attr['label'];

        $args = array(
            'post_type'      => 'post',
            'posts_per_page' => $display_limit,
            'tax_query'      => $slug_array,
            'exclude'        => array( $post_id ),
        );

        $raw_posts = get_posts( $args );
        wp_reset_postdata();

        $posts = array();

        foreach ( $raw_posts as $post_obj ) {
            $posts[] = $this->parse_post( $post_obj );
        }

        $render_params = array(
            'label' => $related_label,
            'limit' => $display_limit,
        );

        $buff = $this->toHTML( $posts, $render_params );

        return $buff;
    }

    /**
     * Generates a nice array with some fields out of WP_Post
     */
    public function parse_post($req_object) {
        $pm_recently_viewed_admin_obj = pm_recently_viewed_admin::get_instance();
        $opts = $pm_recently_viewed_admin_obj->get_options();
        
        $page_rec = array(
            'id' => $req_object->ID,
            'title' => apply_filters( 'the_title', $req_object->post_title, $req_object->ID ),
            'link' => get_permalink( $req_object->ID ),
            'excerpt' => '',
            'featured_image_html' => '',
        );

        // do we need featured_image_html
        if ( in_array( $opts['how_to_render'], array( 'linked_title_with_featured_image', 'linked_title_with_featured_image_and_excerpt' ) )
                && has_post_thumbnail( $req_object->ID ) ) {
            $page_rec['featured_image_html'] = get_the_post_thumbnail( $req_object->ID );
        }

        // do we need the excerpt
        if ( in_array( $opts['how_to_render'], array( 'linked_title_with_excerpt', 'linked_title_with_featured_image_and_excerpt' ) ) ) {
            // @see http://wordpress.stackexchange.com/questions/37618/apply-filters-and-the-excerpt-are-giving-unexpected-results
            $page_rec['excerpt'] = $req_object->post_excerpt;
            $page_rec['excerpt'] = strip_tags( $page_rec['excerpt'] );
            $page_rec['excerpt'] = apply_filters( 'get_the_excerpt', $page_rec['excerpt'] );
        }

        return $page_rec;
    }

    /**
     * This adds the viewed pages in the session.
     * Will we be able to save that info if there is (full page) caching???
     */
    public function track_viewed_page() {
        if ( is_admin() ) {
            return;
        }

        $keep_view_log_limit = 20;
        $req_object = get_queried_object(); // wp 3.1+
        
        // track only 'post' types; if ID is empty this could be the home page.
        if ( empty( $req_object->ID ) || get_post_type( $req_object->ID ) != 'post' || isset( $_REQUEST['clear_pm_recently_viewed_pages'] ) ) {
            return;
        }

        // Does it exist in the list? If so let's remove it and so we can add it to the end.
        $existing_pos = array_search( $req_object->ID, $_SESSION['pm_recently_viewed_pages'] );

        if ( $existing_pos !== false ) {
            unset( $_SESSION[ 'pm_recently_viewed_pages' ][ $existing_pos ] );
        }

        $_SESSION[ 'pm_recently_viewed_pages' ][] = $req_object->ID; // use ID so we don't get duplicates for one page.

        // When we go over the limit we'll remove the oldest one. We can only add one new page.
        if ( count( $_SESSION['pm_recently_viewed_pages'] ) > $keep_view_log_limit ) {
            array_shift( $_SESSION['pm_recently_viewed_pages'] );
        }
    }
}
