// Docu : http://wiki.moxiecode.com/index.php/TinyMCE:Create_plugin/3.x#Creating_your_own_plugins

(function() {
	// Load plugin specific language pack
	tinymce.PluginManager.requireLangPack('pm_recently_viewed');
	tinymce.create('tinymce.plugins.pm_recently_viewed', {
		/**
		 * Initializes the plugin, this will be executed after the plugin has been created.
		 * This call is done before the editor instance has finished it's initialization so use the onInit event
		 * of the editor instance to intercept that event.
		 *
		 * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		 * @param {string} url Absolute URL to where the plugin is located.
		 */
		init : function(ed, url) {
			// Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mceExample');

			ed.addCommand('pm_recently_viewed', function() {
				ed.windowManager.open({
					file : ajaxurl + '?action=pm_recently_viewed_ajax_render_popup_content', // wp admin ajax variable
					width : 600 + ed.getLang('pm_recently_viewed.delta_width', 0),
					height : 400 + ed.getLang('pm_recently_viewed.delta_height', 0),
					inline : 1
				}, {
					plugin_url : url // Plugin absolute URL
				});
			});

			// Register example button
			ed.addButton('pm_recently_viewed', {
				title : 'PM Recently Viewed',
				cmd : 'pm_recently_viewed',
				image : url + '/icon.png'
			});
		},

		/**
		 * Returns information about the plugin as a name/value array.
		 * The current keys are longname, author, authorurl, infourl and version.
		 *
		 * @return {Object} Name/value array containing information about the plugin.
		 */
		getInfo : function() {
			return {
					longname  : 'PM Recently Viewed',
					author 	  : 'Svetoslav Marinov',
					authorurl : 'http://orbisius.com',
					infourl   : 'http://club.orbisius.com/products/wordpress-plugins/',
					version   : "1.0.0"
			};
		}
	});

	// Register plugin
	tinymce.PluginManager.add('pm_recently_viewed', tinymce.plugins.pm_recently_viewed);
})();
